#include "StdAfx.h"
#include "Preference.h"
#include "Utils.h"

////////////////////////////////////////////////////////////////////////////////

const LPCTSTR SETTINGS_SECTION_DESKBAND   = TEXT("Deskband");

const LPCTSTR SETTINGS_KEY_HEIGHT =			TEXT("Height");
const LPCTSTR SETTINGS_KEY_FRAME =			TEXT("Frame");
const LPCTSTR SETTINGS_KEY_MINLENGTH =		TEXT("MinLength");
const LPCTSTR SETTINGS_KEY_ORIENTAL =		TEXT("Oriental");
const LPCTSTR SETTINGS_KEY_POPUP =			TEXT("Popup");
const LPCTSTR SETTINGS_KEY_COLOR1 =			TEXT("Color1");
const LPCTSTR SETTINGS_KEY_COLOR2 =			TEXT("Color2");
const LPCTSTR SETTINGS_KEY_COLORBG =		TEXT("ColorBG");
const LPCTSTR SETTINGS_KEY_COLORFRAME1 =	TEXT("ColorFrame1");
const LPCTSTR SETTINGS_KEY_COLORFRAME2 =	TEXT("ColorFrame2");
const LPCTSTR SETTINGS_KEY_COLORMUTE =		TEXT("ColorMute");
const LPCTSTR SETTINGS_KEY_VOLSTEP =		TEXT("VolStep");
const LPCTSTR SETTINGS_KEY_IGNOREFIRSTCLICK=TEXT("IgnoreFirstClick");
const LPCTSTR SETTINGS_KEY_MUTEDOUBLECLICK =TEXT("MuteDoubleClick");
const LPCTSTR SETTINGS_KEY_TARGET =			TEXT("Target");

const LONG SETTINGS_DEF_HEIGHT =	16;
const LONG SETTINGS_DEF_FRAME =		3;
const LONG SETTINGS_DEF_MINLENGTH =	80;
const LONG SETTINGS_DEF_ORIENTAL =	0;
const LONG SETTINGS_DEF_POPUP =		1;
const LONG SETTINGS_DEF_COLOR1 =	0xFF00FF00;
const LONG SETTINGS_DEF_COLOR2 =	0xFF0000FF;
const LONG SETTINGS_DEF_GRADIETION =	1;
const LONG SETTINGS_DEF_COLORBG =	0xFF000000;
const LONG SETTINGS_DEF_COLORFRAME1 =	0xFFC0C0C0;
const LONG SETTINGS_DEF_COLORFRAME2 =	0xFF808080;
const LONG SETTINGS_DEF_COLORMUTE =	0xFFFF0000;
const LONG SETTINGS_DEF_VOLSTEP =	1;
const LONG SETTINGS_DEF_IGNOREFIRSTCLICK=1;
const LONG SETTINGS_DEF_MUTEDOUBLECLICK =1;
const LONG SETTINGS_DEF_TARGET =	0;

////////////////////////////////////////////////////////////////////////////////

static	bool EnsureFolderPathExists(LPCTSTR pszPath)
{
    CPath pathFolder(pszPath);
    pathFolder.RemoveFileSpec();

    if(pathFolder.IsDirectory())
        return true;

    ATLASSERT(!pathFolder.FileExists());

    const int nRes = ::SHCreateDirectoryEx(NULL, pathFolder, NULL);
    ATLASSERT(nRes == ERROR_SUCCESS);

    return (nRes == ERROR_SUCCESS);
}


CPreference::CPreference(void)
{
}

CPreference::~CPreference(void)
{
}

////////////////////////////////////////////////////////////////////////////////

void CPreference::Load(CPath pathSettings)
{
	LPCTSTR pszFilename = pathSettings;

#define LOAD( id )	(::GetPrivateProfileInt(SETTINGS_SECTION_DESKBAND, SETTINGS_KEY_##id, SETTINGS_DEF_##id , pszFilename) )
#define LOADB( id ) ( LOAD(id) != 0 );

	m_Height		=LOAD(HEIGHT);
	m_Frame			=LOAD(FRAME);
	m_MinLength		=LOAD(MINLENGTH);
	m_Oriental		=LOAD(ORIENTAL);
	m_Popup			=LOADB(POPUP);
	m_Color1		=LOAD(COLOR1);
	m_Color2		=LOAD(COLOR2);
	m_ColorBG		=LOAD(COLORBG);
	m_ColorFrame1		=LOAD(COLORFRAME1);
	m_ColorFrame2		=LOAD(COLORFRAME2);
	m_ColorMute		=LOAD(COLORMUTE);
	m_VolStep		=LOAD(VOLSTEP);
	m_IgnoreFirstClick	=LOADB(IGNOREFIRSTCLICK);
	m_MuteDoubleClick	=LOADB(MUTEDOUBLECLICK);
	m_Target			=LOAD(TARGET);


	//m_bDeskband = (::GetPrivateProfileInt(
 //       SETTINGS_SECTION_APPEARANCE, SETTINGS_KEY_DESKBAND,
 //       FALSE, pszFilename) != 0);

#undef LOADB
#undef LOAD

}

bool CPreference::Save(CPath pathSettings) const
{
	LPCTSTR pszFilename = pathSettings;

	if(!::EnsureFolderPathExists(pszFilename))
        return false;

    BOOL bWrite = FALSE;

	TCHAR	buff[256];
#define SAVE( id,val  )	{ _stprintf( buff , _T("%d") , val ); bWrite= ::WritePrivateProfileString( SETTINGS_SECTION_DESKBAND, SETTINGS_KEY_##id,buff, pszFilename); if (!bWrite) return false;}

	SAVE( HEIGHT,			m_Height);
	SAVE( FRAME,                  m_Frame);
	SAVE( MINLENGTH,              m_MinLength);
	SAVE( ORIENTAL,               m_Oriental);
	SAVE( POPUP,                  m_Popup);
	SAVE( COLOR1,                 m_Color1);
	SAVE( COLOR2,                 m_Color2);
	SAVE( COLORBG,                m_ColorBG);
	SAVE( COLORFRAME1,             m_ColorFrame1);
	SAVE( COLORFRAME2,             m_ColorFrame2);
	SAVE( COLORMUTE,             m_ColorMute);
	SAVE( VOLSTEP,                m_VolStep);
	SAVE( IGNOREFIRSTCLICK,       m_IgnoreFirstClick);
	SAVE( MUTEDOUBLECLICK,        m_MuteDoubleClick);
	SAVE( TARGET,                 m_Target);

#undef SAVE

    return true;
}
