#include "StdAfx.h"
#include "VolumeBar.h"

////////////////////////////////////////////////////////////////////////////////
// 

class DoubleBufferPaint
{
public:
    DoubleBufferPaint(HWND hWnd, DWORD dwRop = SRCCOPY) : m_hWnd(hWnd), m_dwRop(dwRop)
    {
        m_hdc = ::BeginPaint(m_hWnd, &m_ps);

        int x, y, cx, cy;
        InitDims(&x, &y, &cx, &cy);

        m_hdcMem = ::CreateCompatibleDC(m_hdc);
        m_hbmMem = ::CreateCompatibleBitmap(m_hdc, cx, cy);
        m_hbmSave = ::SelectObject(m_hdcMem, m_hbmMem);
        ::SetWindowOrgEx(m_hdcMem, x, y, NULL);
    }

    ~DoubleBufferPaint()
    {
        int x, y, cx, cy;
        InitDims(&x, &y, &cx, &cy);

        ::BitBlt(m_hdc, x, y, cx, cy, m_hdcMem, x, y, m_dwRop);

        ::SelectObject(m_hdcMem, m_hbmSave);
        ::DeleteObject(m_hbmMem);
        ::DeleteDC(m_hdcMem);

        ::EndPaint(m_hWnd, &m_ps);
    }

    HDC GetDC() const { return m_hdcMem; }
    const RECT& GetPaintRect() const { return m_ps.rcPaint; }

private:
    void InitDims(int* px, int* py, int* pcx, int* pcy) const
    {
        *px = m_ps.rcPaint.left;
        *py = m_ps.rcPaint.top;
        *pcx = m_ps.rcPaint.right - m_ps.rcPaint.left;
        *pcy = m_ps.rcPaint.bottom - m_ps.rcPaint.top;
    }

private:
    HWND m_hWnd;
    DWORD m_dwRop;

    HDC m_hdc;
    HDC m_hdcMem;
    HGDIOBJ m_hbmMem;
    HGDIOBJ m_hbmSave;
    PAINTSTRUCT m_ps;

private:
    // not copyable
    DoubleBufferPaint(const DoubleBufferPaint&);
    DoubleBufferPaint& operator=(const DoubleBufferPaint&);
};

/////////////////////////////////////////////////////////////////////////////

CVolumeBar::CVolumeBar(void):
    m_ptrVisualStyle(CVisualStyle::Create())
{
	GdiplusStartupInput gdiSI;
	GdiplusStartup(&mGdiToken, &gdiSI, NULL);
}

CVolumeBar::~CVolumeBar(void)
{
	GdiplusShutdown(mGdiToken);
}

BOOL CVolumeBar::Create(
    HWND hwndParent,
	const   CPreference* pPrefs,
	CStatus	*pStat)
{
    if(!__super::Create(hwndParent))
        return FALSE;

	m_hwndParent = hwndParent;

    ATLASSERT(pPrefs);
    m_pPrefs = pPrefs;

    ATLASSERT(pStat);
    m_pStat= pStat;

	m_1stclick = true;

    return TRUE;
}

LRESULT CVolumeBar::OnDestroy(
    UINT /*uMsg*/,
    WPARAM /*wParam*/,
    LPARAM /*lParam*/,
    BOOL& /*bHandled*/)
{

    return 0;
}


LRESULT CVolumeBar::OnThemeChanged(
    UINT /*uMsg*/,
    WPARAM /*wParam*/,
    LPARAM /*lParam*/,
    BOOL& /*bHandled*/)
{
    // re-create theme style
    m_ptrVisualStyle.Free();
    m_ptrVisualStyle.Attach(CVisualStyle::Create());

    return 0;
}
LRESULT CVolumeBar::OnEraseBackground(
    UINT /*uMsg*/,
    WPARAM /*wParam*/,
    LPARAM /*lParam*/,
    BOOL& /*bHandled*/)
{
    return 1;
}

LRESULT CVolumeBar::OnPaint(
    UINT /*uMsg*/,
    WPARAM /*wParam*/,
    LPARAM /*lParam*/,
    BOOL& /*bHandled*/)
{
    DoubleBufferPaint dbuffPaint(m_hWnd);
    
    Paint(dbuffPaint.GetDC(), dbuffPaint.GetPaintRect());

    return 0;
}

void CVolumeBar::Paint(HDC hdc, const RECT& rcPaint) const
{
    m_ptrVisualStyle->DrawBackground(m_hwndParent, hdc, rcPaint);
	

    RECT rcClient = { 0 };
    GetClientRect(&rcClient);

	const int wmergin = 2;
	const int topmergin = 2;

	int VCenter = (rcClient.bottom+(rcClient.top+topmergin))/2;


	RectF rect( 
		(REAL)(rcClient.left+wmergin) , 
		(REAL)(VCenter - m_pPrefs->m_Height/2 ) , 
		(REAL)(rcClient.right-rcClient.left-wmergin*2),
		(REAL)( m_pPrefs->m_Height )
		);
	
	SizeF sz( 8,8 );

	DrawBar( hdc , rect , sz );

	m_pStat->SetBarPos( (int)rect.GetLeft() ,  (int)rect.GetRight() );
}


void CVolumeBar::CreateRoundedRectangle(GraphicsPath &path , RectF rect, SizeF size) const
{
    path.AddLine(rect.GetLeft() + size.Width / 2, rect.GetTop(),
                       rect.GetRight() - size.Width / 2, rect.GetTop());
    path.AddArc(rect.GetRight() - size.Width, rect.GetTop(),
                      size.Width, size.Height, 270, 90);
    path.AddLine(rect.GetRight(), rect.GetTop() + size.Height / 2,
                       rect.GetRight(), rect.GetBottom() - size.Height / 2);
    path.AddArc(rect.GetRight() - size.Width,
                      rect.GetBottom() - size.Height,
                      size.Width, size.Height, 0, 90);
    path.AddLine(rect.GetRight() - size.Width / 2, rect.GetBottom(),
                       rect.GetLeft() + size.Width / 2, rect.GetBottom());
    path.AddArc(rect.GetLeft(), rect.GetBottom() - size.Height,
                      size.Width, size.Height, 90, 90);
    path.AddLine(rect.GetLeft(), rect.GetBottom() - size.Height / 2,
                       rect.GetLeft(), rect.GetTop() + size.Height / 2);
    path.AddArc(rect.GetLeft(), rect.GetTop(),
                      size.Width, size.Height, 180, 90);

}

void CVolumeBar::DrawBar(  HDC hdc , RectF barRect, SizeF barRound ) const
{
	Graphics gr( hdc );
	GraphicsPath path;

	int	volume = m_pStat->GetVolume();
	bool Mute = m_pStat->GetMute();
	gr.SetSmoothingMode( SmoothingMode::SmoothingModeAntiAlias );
    // {^{
	{
		RectF rect = barRect;

		SizeF round = barRound;

		path.Reset();
		CreateRoundedRectangle(path ,rect, round);

		// wi
		if ( ( m_pPrefs->m_ColorBG & 0xFF000000 )!=0  ){
			SolidBrush brs( m_pPrefs->m_ColorBG );
			gr.FillPath( &brs, &path );
		}

		// {
		if ( volume > 0 ){
			RectF rect = barRect;
			rect.Width = rect.Width * volume / 100;

			path.Reset();
			CreateRoundedRectangle(path ,rect, round);

			Point	start((int)barRect.GetLeft(), (int)barRect.GetTop());
			Point	end((int)barRect.GetRight(), (int)barRect.GetBottom());
			LinearGradientBrush brs( start , end , m_pPrefs->m_Color1 , m_pPrefs->m_Color2);

			gr.FillPath( &brs, &path );
		}
		//// ̉e
		//  // Path̓{^Ɠ̂g
		//{
		//	Point	start((int)rect.GetLeft(), (int)rect.GetTop());
		//	Point	end((int)rect.GetLeft(), (int)rect.GetBottom());
		//	LinearGradientBrush brs(	start,  end,  0, 0 );

		//	Color  col[]  = { 0x00000000 , 0x00000000 , 0x80000000 };
		//	REAL   pts[]  = { 0 , 0.50F , 1.0F };
		//	brs.SetInterpolationColors( col , pts , 3 );
		//	gr.FillPath( &brs, &path );
		//}
	}
    // Og
	{

		RectF rect = barRect;

		SizeF round = barRound;

		path.Reset();
		CreateRoundedRectangle( path , rect , round);

		Point	start((int)barRect.GetLeft()-1, (int)barRect.GetTop()-1);
		Point	end((int)barRect.GetRight()+1, (int)barRect.GetBottom()+1);
		LinearGradientBrush brs( start , end  , m_pPrefs->m_ColorFrame1 , m_pPrefs->m_ColorFrame2 );
		//brs.SetWrapMode( WrapMode::WrapModeClamp );
		Pen pen( &brs , (REAL)m_pPrefs->m_Frame );

		gr.DrawPath( &pen , &path );
	}

//      // 
	//{
	//	RectF rect = barRect;
	//	rect.Y += wmergin;
	//	rect.X += hmergin;
	//	rect.Height = (REAL)(int)(rect.Height/2);
	//	rect.Width -= wmergin*2;

	//	SizeF round = barRound;
	//	//round.Width = rect.Height;
	//	//round.Height = rect.Height;

	//	GraphicsPath path;
	//	CreateRoundedRectangle( path , rect, round );

	//	Point	start((int)rect.GetLeft(), (int)rect.GetTop());
	//	Point	end((int)rect.GetLeft(), (int)rect.GetBottom());
	//	LinearGradientBrush brs ( start , end, 0x70FFFFFF , 0x08FFFFFF );
	//	gr.FillPath( &brs , &path);
	//}
	if ( Mute ){
		RectF rect = barRect;

		Font visualfont( hdc , m_ptrVisualStyle->GetFont() );
		LOGFONT	logfont;
		visualfont.GetLogFontW( &gr , &logfont );


		path.Reset();
		PointF	pt(0,0);

		StringFormat	fmt;
		//fmt.SetAlignment( StringAlignment::StringAlignmentCenter );
		//fmt.SetLineAlignment( StringAlignment::StringAlignmentCenter );

		SolidBrush brs( m_pPrefs->m_ColorMute );

		FontFamily fam( logfont.lfFaceName );
		path.AddString( _T("MUTE"),4 , &fam , FontStyle::FontStyleBold , (REAL)(m_pPrefs->m_Height -  m_pPrefs->m_Frame ), pt , &fmt );

		RectF fontrect;
		path.GetBounds( &fontrect );  

		Matrix mtx;
		mtx.Translate(  (rect.X - fontrect.X) + (rect.Width - fontrect.Width)/2 , 
						(rect.Y - fontrect.Y) + (rect.Height - fontrect.Height)/2 );
		path.Transform( &mtx );
		gr.FillPath( &brs , &path );
		//gr.DrawString( _T("MUTE") ,4 , &font , barRect , &fmt , &brs );

	}

}

LRESULT CVolumeBar::OnLButtonDblClk(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if ( m_pPrefs->m_MuteDoubleClick ){
		m_pStat->SetMute(m_pStat->GetMute()==false);
		Invalidate();
		m_1stclick = true;
	}
	return 0;
}

LRESULT CVolumeBar::OnMouseWheel(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	short delta = (short)(HIWORD(wParam));

	delta /= WHEEL_DELTA;

	int volume = m_pStat->GetVolume();

	volume += (delta*m_pPrefs->m_VolStep);

	if ( volume > 100 ){
		volume = 100 ;
	}

	if ( volume < 0 ){
		volume = 0 ;
	}

	m_pStat->SetVolume(volume);

	Invalidate();

	return 0;
}

LRESULT CVolumeBar::OnLButtonDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
{
	if ( SetFocus() != m_hWnd || m_1stclick ){
		m_1stclick = false;
		if ( m_pPrefs->m_IgnoreFirstClick ){
			return 0;
		}
	}

	int xPos = LOWORD(lParam);

	int	low = m_pStat->GetPosLow();
	int	high = m_pStat->GetPosHigh();

	if ( low <= xPos && xPos <= high ){
		m_pStat->SetVolume( 100 * ( xPos - low)  / ( high - low ) );
		Invalidate();
	}


	return 0;
}

