#pragma once

#include <functiondiscoverykeys.h>
#include <Mmdeviceapi.h>
#include <EndpointVolume.h>
#include "IVolumeControl.h"

template <class T> void SafeRelease(T **ppT)
{
    if (*ppT)
    {
        (*ppT)->Release();
        *ppT = NULL;
    }
}


class CVolumeControl : 	public IVolumeControl , public IAudioEndpointVolumeCallback
{
private:
	IMMDevice *m_device ;
	IAudioEndpointVolume *m_endpointVolume;
	IVolumeControlCallback *m_changed;
    LONG m_RefCount; 

public:
	CVolumeControl(DeviceRole role=drConsole,IVolumeControlCallback *cbfunc=NULL): m_RefCount(1) 
	{
	    HRESULT hr;
		IMMDeviceEnumerator *deviceEnumerator = NULL;
		m_device = NULL;
		m_endpointVolume=NULL;

		m_changed = cbfunc;

		hr = CoCreateInstance(__uuidof(MMDeviceEnumerator), NULL, CLSCTX_INPROC_SERVER, IID_PPV_ARGS(&deviceEnumerator));
		if (FAILED(hr))	{
			goto Exit;
		}

        ERole deviceRole = eConsole;    // Assume we're using the console role.
		switch( role ){
		case drConsole:
			deviceRole = eConsole;
			break;
		case drComm:
			deviceRole = eCommunications;
			break;
		case drMM:
			deviceRole = eMultimedia;
			break;
		}
        hr = deviceEnumerator->GetDefaultAudioEndpoint(eRender, deviceRole, &m_device);
        if (FAILED(hr)){
            goto Exit;
        }

		hr = m_device->Activate(__uuidof(IAudioEndpointVolume), CLSCTX_INPROC_SERVER, NULL, reinterpret_cast<void **>(&m_endpointVolume));
		if (FAILED(hr)){
			goto Exit;
		}
		if ( m_changed ){
			m_endpointVolume->RegisterControlChangeNotify( this );
		}

Exit:
		SafeRelease( &deviceEnumerator );
	}
	virtual ~CVolumeControl(void)
	{
		if ( m_endpointVolume && m_changed ){
			m_endpointVolume->UnregisterControlChangeNotify( this );
			
		}
		SafeRelease( &m_endpointVolume );
		SafeRelease( &m_device );
	}

public:	// IVolumeControl
	void	SetVolume( int volume ){
		if ( m_endpointVolume ){
			float  newVolume = volume / 100.0F;
			m_endpointVolume->SetMasterVolumeLevelScalar(newVolume, NULL);
		}
	}
	int	GetVolume( ){
		if ( m_endpointVolume ){
			float  newVolume;
			m_endpointVolume->GetMasterVolumeLevelScalar(&newVolume);
			return (int)(newVolume*100+0.5F);		// ľܓ
		}
		return 0;
	}
	void	SetMute( bool mute ){
		if ( m_endpointVolume ){
			m_endpointVolume->SetMute( mute?TRUE:FALSE , NULL );
		}
	}

	bool GetMute(){
		if ( m_endpointVolume ){
			BOOL  mute;
			m_endpointVolume->GetMute(&mute);
			return mute!=FALSE;
		}
		return false;
	}
public:		// IUnknown
    STDMETHODIMP_(ULONG)AddRef()
	{ 
		return InterlockedIncrement(&m_RefCount); 
	} 
    STDMETHODIMP_(ULONG)Release()  
    { 
        LONG ref = InterlockedDecrement(&m_RefCount);  
        if (ref == 0) 
            delete this; 
        return ref; 
    } 
    STDMETHODIMP QueryInterface(REFIID IID, void **ReturnValue) 
    { 
        if (IID == IID_IUnknown || IID== __uuidof(IAudioEndpointVolumeCallback))  
        { 
            *ReturnValue = static_cast<IUnknown*>(this); 
            AddRef(); 
            return S_OK; 
        } 
        *ReturnValue = NULL; 
        return E_NOINTERFACE; 
    } 
public:	//IAudioEndpointVolumeCallback 
	virtual HRESULT STDMETHODCALLTYPE OnNotify( /* [in]*/  PAUDIO_VOLUME_NOTIFICATION_DATA pNotify)
	{
		if ( m_changed ){
			m_changed->onVolumeChanged( (int)(pNotify->fMasterVolume * 100) , pNotify->bMuted!=FALSE );
		}
		return S_OK;
	}



};

