// VolumeDeskBand.h : Declaration of the CVolumeDeskBand

#pragma once

#include "VolumeGuids.h"
#include "VolumeWindow.h"

#include "Preference.h"
#include "IVolumeControl.h"

#if defined(_WIN32_WCE) && !defined(_CE_DCOM) && !defined(_CE_ALLOW_SINGLE_THREADED_OBJECTS_IN_MTA)
#error "Single-threaded COM objects are not properly supported on Windows CE platform, such as the Windows Mobile platforms that do not include full DCOM support. Define _CE_ALLOW_SINGLE_THREADED_OBJECTS_IN_MTA to force ATL to support creating single-thread COM object's and allow use of it's single-threaded COM object implementations. The threading model in your rgs file was set to 'Free' as that is the only threading model supported in non DCOM Windows CE platforms."
#endif

using namespace ATL;

// CVolumeDeskBand

class CVolumeDeskBand :
    public CComObjectRootEx<CComSingleThreadModel>,
    public CComCoClass<CVolumeDeskBand, &CLSID_VolumeDeskBand>,
    public IObjectWithSiteImpl<CVolumeDeskBand>,
    public IPersistStreamInitImpl<CVolumeDeskBand>,
    public IInputObject,
    public IContextMenu,
    public IDeskBand2,
	public IVolumeControlCallback
{
    typedef IPersistStreamInitImpl<CVolumeDeskBand> IPersistStreamImpl;
public:
    CVolumeDeskBand();

DECLARE_REGISTRY_RESOURCEID(IDR_VOLUMEDESKBAND)

BEGIN_COM_MAP(CVolumeDeskBand)
    COM_INTERFACE_ENTRY(IOleWindow)
    COM_INTERFACE_ENTRY(IDockingWindow)
    COM_INTERFACE_ENTRY(IDeskBand)
    COM_INTERFACE_ENTRY(IDeskBand2)
    COM_INTERFACE_ENTRY(IInputObject)
    COM_INTERFACE_ENTRY(IContextMenu)
    COM_INTERFACE_ENTRY(IObjectWithSite)
    COM_INTERFACE_ENTRY_IID(IID_IPersist, IPersistStreamImpl)
    COM_INTERFACE_ENTRY_IID(IID_IPersistStream, IPersistStreamImpl)
    COM_INTERFACE_ENTRY_IID(IID_IPersistStreamInit, IPersistStreamImpl)
END_COM_MAP()

BEGIN_CATEGORY_MAP(CVolumeDeskBand)
    IMPLEMENTED_CATEGORY(CATID_DeskBand)
END_CATEGORY_MAP()

// IPersistStreamInitImpl requires property map.
BEGIN_PROP_MAP(CVolumeDeskBand)
    //PROP_DATA_ENTRY("Locale", m_dateFormat.lcId, VT_UI4)
    //PROP_DATA_ENTRY("Calendar", m_dateFormat.calId, VT_UI4)
    //PROP_DATA_ENTRY("CalendarType", m_dateFormat.calType, VT_UI4)
    //PROP_DATA_ENTRY("DateFormat", m_bstrDateFormat.m_str, VT_BSTR)
END_PROP_MAP()

    DECLARE_PROTECT_FINAL_CONSTRUCT()

    HRESULT FinalConstruct();
    void FinalRelease();

public:
    // IObjectWithSite
    //
    STDMETHOD(SetSite)( 
        /* [in] */ IUnknown *pUnkSite);

    // IInputObject
    //
    STDMETHOD(UIActivateIO)( 
        /* [in] */ BOOL fActivate,
        /* [unique][in] */ MSG *pMsg);
        
    STDMETHOD(HasFocusIO)();

    STDMETHOD(TranslateAcceleratorIO)( 
        /* [in] */ MSG *pMsg);

    // IContextMenu
    //
    STDMETHOD(QueryContextMenu)(
        /* [in] */ HMENU hmenu,
        /* [in] */ UINT indexMenu,
        /* [in] */ UINT idCmdFirst,
        /* [in] */ UINT idCmdLast,
        /* [in] */ UINT uFlags);

    STDMETHOD(InvokeCommand)( 
        /* [in] */ CMINVOKECOMMANDINFO *pici);

    STDMETHOD(GetCommandString)( 
        /* [in] */ UINT_PTR idCmd,
        /* [in] */ UINT uType,
        /* [in] */ UINT *pReserved,
        /* [out] */ LPSTR pszName,
        /* [in] */ UINT cchMax);

    // IDeskBand
    //
    STDMETHOD(GetBandInfo)(
        /* [in] */ DWORD dwBandID,
        /* [in] */ DWORD dwViewMode,
        /* [out][in] */ DESKBANDINFO *pdbi);

    // IDeskBand2
    //
    STDMETHOD(CanRenderComposited)( 
        /* [out] */ BOOL *pfCanRenderComposited);
        
    STDMETHOD(SetCompositionState)( 
        /* [in] */ BOOL fCompositionEnabled);
        
    STDMETHOD(GetCompositionState)( 
        /* [out] */ BOOL *pfCompositionEnabled);
    
    // IOleWindow
    //
    STDMETHOD(GetWindow)( 
        /* [out] */ HWND *phwnd);

    STDMETHOD(ContextSensitiveHelp)( 
        /* [in] */ BOOL fEnterMode);

    // IDockingWindow
    //
    STDMETHOD(ShowDW)( 
        /* [in] */ BOOL fShow);

    STDMETHOD(CloseDW)( 
        /* [in] */ DWORD dwReserved);

    STDMETHOD(ResizeBorderDW)( 
        /* [in] */ LPCRECT prcBorder,
        /* [in] */ IUnknown *punkToolbarSite,
        /* [in] */ BOOL fReserved);

    // IPersistStreamImpl
    //
    HRESULT IPersistStreamInit_Load(
        /* [in] */ LPSTREAM pStm,
        /* [in] */ ATL_PROPMAP_ENTRY* pMap);

    HRESULT IPersistStreamInit_Save(
        /* [in] */ LPSTREAM pStm,
        /* [in] */ BOOL fClearDirty,
        /* [in] */ ATL_PROPMAP_ENTRY* pMap);

	void onVolumeChanged( int vol , bool mute );
private:
    HRESULT HandleShowRequest();
    HRESULT UpdateDeskband();
	void	SetVolumeTarget(int target);

public:
    bool m_bRequiresSave; // used by IPersistStreamInitImpl

private:
    int m_nBandID;
    DWORD m_dwViewMode;
    //DateFormat m_dateFormat;
    CVolumeWindow m_wndVolume;
    //CComBSTR m_bstrDateFormat;  // for persistence only [used in property map]
    BOOL m_bCompositionEnabled;
    CPath m_settingsPath;

	CPreference	m_preference;
	CStatus	m_Stat;
	IVolumeControl *m_pVolCtrl;
};

OBJECT_ENTRY_AUTO(CLSID_VolumeDeskBand, CVolumeDeskBand)
