// VDSetup.cpp : Implementation of CVDSetup

#include "StdAfx.h"
#include "VDSetup.h"
#include "Volumeguids.h"
#include "utils.h"
#include "commonversion.h"

const LPCTSTR SETTINGS_SECTION_APPEARANCE   = TEXT("Appearance");
const LPCTSTR SETTINGS_KEY_DESKBAND     = TEXT("Deskband");

////////////////////////////////////////////////////////////////////////////////

static	bool EnsureFolderPathExists(LPCTSTR pszPath)
{
    CPath pathFolder(pszPath);
    pathFolder.RemoveFileSpec();

    if(pathFolder.IsDirectory())
        return true;

    ATLASSERT(!pathFolder.FileExists());

    const int nRes = ::SHCreateDirectoryEx(NULL, pathFolder, NULL);
    ATLASSERT(nRes == ERROR_SUCCESS);

    return (nRes == ERROR_SUCCESS);
}

/////////////////////////////////////////////////////////////////////////////
//
static const LPCTSTR DESKBAND_BIN_NAME = TEXT("VolumeDeskBand.dll");

////////////////////////////////////////////////////////////////////////////////
// 
static	CString StringFromGUID2(REFGUID rguid)
{
    OLECHAR szGuid[MAX_GUID_STRING_LEN] = { 0 };
    ::StringFromGUID2(rguid, szGuid, MAX_GUID_STRING_LEN);

    return szGuid;
}

static DWORD RunAsAdmin(HWND hWnd, LPCTSTR lpFile, LPCTSTR lpParameters)
{
    SHELLEXECUTEINFO sei = { 0 };

    const ULONG fMask =
        SEE_MASK_FLAG_DDEWAIT |
        SEE_MASK_FLAG_NO_UI |
        SEE_MASK_NOCLOSEPROCESS;

    sei.cbSize          = sizeof(SHELLEXECUTEINFOW);
    sei.hwnd            = hWnd;
    sei.fMask           = fMask;
    sei.lpVerb          = TEXT("runas");
    sei.lpFile          = lpFile;
    sei.lpParameters    = lpParameters;
    sei.nShow           = SW_SHOWNORMAL;

    if(::ShellExecuteEx(&sei))
    {
        CHandle handle(sei.hProcess);
        const DWORD dwWaitRes = ::WaitForSingleObject(handle, INFINITE);

        if(dwWaitRes == WAIT_OBJECT_0)
        {
            DWORD dwExitCode = ERROR_SUCCESS;
            const BOOL bRes = ::GetExitCodeProcess(handle, &dwExitCode);
            ATLASSERT(bRes);

            return dwExitCode;
        }
    }

    return ::GetLastError();
}


////////////////////////////////////////////////////////////////////////////////
// CVDSetup

CVDSetup::CVDSetup()
{
}

CVDSetup::~CVDSetup()
{
}

LRESULT CVDSetup::OnInitDialog(
    UINT /*uMsg*/,
    WPARAM /*wParam*/,
    LPARAM lParam,
    BOOL& /*bHandled*/)
{
    if(lParam)
    {
    }

	if ( !InitSettingsPath() ) 
		return 0;

	LoadSettings();

	// CXg[ς݂ǂɊ֌WȂACXg[Ƀ`FbN
	HWND hRadio = GetDlgItem( IDC_RADIOINSTALL );
	SendMessage( hRadio, BM_SETCHECK, BST_CHECKED, 0 );  // ON


    m_link.SubclassWindow(GetDlgItem(IDC_VERSION));
	m_link.SetLabel(_T("AhGXL http://pandora.sblo.jp"));
    m_link.SetHyperLink(_T("http://pandora.sblo.jp/article/34433147.html"));


	SYSTEM_INFO	si;
	GetSystemInfo(&si);

	TCHAR	title[256];
	::GetWindowText( m_hWnd , title , 240 );

	if ( si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL ){
		// x86 32bit
		_tcscat( title , TEXT(" for x86/32bit") );
	}else if ( si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_AMD64 ){
		_tcscat( title , _TEXT(" for x64") );
	}else if ( si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_IA64 ){
		_tcscat( title , _TEXT(" for IA64") );
	}

	TCHAR	version[256];
	_stprintf( version , _T( " Version %d.%d.%d.%d" ), VER0 ,VER1 ,VER2 ,VER3 );
	_tcscat( title , version );

	::SetWindowText( m_hWnd , title );

	// 32on64 check
    BOOL fWow64 = TRUE;
    ::IsWow64Process(::GetCurrentProcess(), &fWow64);
	if(fWow64 || !IsVistaOrHigher() ) { 
		// Windows XPȑO
		// 64bitOS32bitvZX삵Ă鎞́AG[

		// OK{^fBZ[u
		HWND hOKbtn = GetDlgItem( IDOK );
		::EnableWindow(  hOKbtn , FALSE );

		// bZ[W\
		HWND hErr = GetDlgItem( IDC_ERRORMSG );
		::ShowWindow(  hErr , TRUE );

		return 1;
	}

//	ApplyDeskbandProperties();

	return 1;  // Let the system set the focus
}

LRESULT CVDSetup::OnClickedOK(
    WORD /*wNotifyCode*/,
    WORD wID,
    HWND /*hWndCtl*/,
    BOOL& /*bHandled*/)
{

	HWND hRadio = GetDlgItem( IDC_RADIOINSTALL );
	LRESULT val = SendMessage( hRadio, BM_GETCHECK, BST_CHECKED, 0 );

	m_bDeskband = ( val != 0 );

	ApplyDeskbandProperties();

	SaveSettings();

	EndDialog(wID);
	return 0;
}

LRESULT CVDSetup::OnClickedCancel(
    WORD /*wNotifyCode*/,
    WORD wID,
    HWND /*hWndCtl*/,
    BOOL& /*bHandled*/)
{

	EndDialog(wID);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////
bool CVDSetup::InitSettingsPath()
{
	// exeƓꏊ exe.ini΂D悷B
	// ΁A[U[zini쐬

    ATLASSERT(m_pathSettings.m_strPath.IsEmpty());

    // Try our folder
    TCHAR sz[MAX_PATH] = { TEXT('\0') };
    ::GetModuleFileName(NULL, sz, MAX_PATH);

    CPath pathFilename(sz);
    pathFilename.RenameExtension(TEXT(".ini"));

    if(pathFilename.FileExists())
        m_pathSettings = pathFilename;
    else
        ATLVERIFY(::MakeSettingsFilename(m_pathSettings));

    return (!m_pathSettings.m_strPath.IsEmpty());
}


void CVDSetup::LoadSettings()
{
	LPCTSTR pszFilename = m_pathSettings;

    m_bDeskband = (::GetPrivateProfileInt(
        SETTINGS_SECTION_APPEARANCE, SETTINGS_KEY_DESKBAND,
        FALSE, pszFilename) != 0);

}

bool CVDSetup::SaveSettings() const
{
	LPCTSTR pszFilename = m_pathSettings;

	if(!::EnsureFolderPathExists(pszFilename))
        return false;

    BOOL bWrite = FALSE;


    bWrite = ::WritePrivateProfileString(
        SETTINGS_SECTION_APPEARANCE, SETTINGS_KEY_DESKBAND,
        (m_bDeskband ? TEXT("1") : TEXT("0")), pszFilename);
    ATLASSERT(bWrite);

    if(!bWrite)
        return false;


    return true;
}

////////////////////////////////////////////////////////////////////////////////

bool CVDSetup::ApplyDeskbandProperties() const
{

    const CPath& deskbandPath = GetDeskbandModulePath();
    const bool bDeskbandInstalled = DeskbandInstalled();

    if(m_bDeskband && !bDeskbandInstalled)
    {   // install and show
        if(!ExtractDeskbandBinary(deskbandPath))
            return false;

        if(!RegisterDeskband(deskbandPath, true))
            return false;

        // Show the desk band only if wasn't previously installed;
        // otherwise leave the choice to user.
        if(!ShowDeskband())
            return false;
    }
    else if(!m_bDeskband && bDeskbandInstalled)
    {   // hide and uninstall

        // If there are leftovers of previous deskband objects that cannot
        // be uninstalled (missing files, etc), continue anyway.
        // No point to fail here.
        HideDeskband();
        RegisterDeskband(deskbandPath, false);
        DeleteDeskbandBinary(deskbandPath);
    }

    return true;
}

bool CVDSetup::DeskbandInstalled() const
{
    const CString& keyName = TEXT("CLSID\\") +
        ::StringFromGUID2(CLSID_VolumeDeskBand);

    CRegKey keyDeskband;
    const LONG lRes = keyDeskband.Open(HKEY_CLASSES_ROOT, keyName, KEY_READ);

    return (lRes == ERROR_SUCCESS);
}

CPath CVDSetup::GetDeskbandModulePath() const
{
    CPath deskbandPath = m_pathSettings;
    deskbandPath.RemoveFileSpec();
    deskbandPath.Append(DESKBAND_BIN_NAME);

    return deskbandPath;
}

bool CVDSetup::ExtractDeskbandBinary(const CPath& deskbandPath) const
{
    HRSRC hResInfo = reinterpret_cast<HRSRC>(
        ::FindResource(
            NULL,
            MAKEINTRESOURCE(IDR_DESKBAND_BIN),
            TEXT("DESKBAND")
        )
    );
    ATLASSERT(hResInfo);

    if(!hResInfo) return false;

    HGLOBAL hResData = ::LoadResource(NULL, hResInfo);
    ATLASSERT(hResData);

    if(!hResData) return false;

    LPVOID pDeskbandBinData = ::LockResource(hResData);
    ATLASSERT(pDeskbandBinData);

    if(!pDeskbandBinData) return false;

    const DWORD dwDeskbandBinSize = ::SizeofResource(NULL, hResInfo);
    ATLASSERT(dwDeskbandBinSize > 0);

    if(!dwDeskbandBinSize) return false;

    CAtlFile deskbandBin;
    HRESULT hr = deskbandBin.Create(deskbandPath,
        GENERIC_WRITE, FILE_SHARE_READ, CREATE_ALWAYS);
    ATLASSERT(SUCCEEDED(hr));

    if(FAILED(hr)) return false;

    hr = deskbandBin.Write(pDeskbandBinData, dwDeskbandBinSize);
    ATLASSERT(SUCCEEDED(hr));
    if(FAILED(hr)) return false;

    return true;
}

bool CVDSetup::DeleteDeskbandBinary(const CPath& deskbandPath) const
{
    if(!deskbandPath.FileExists()) return true;

    CPath deskbandDir = deskbandPath;
    deskbandDir.RemoveFileSpec();

    TCHAR szTmpFile[MAX_PATH + 1] = { 0 };
    const UINT nRet = ::GetTempFileName(deskbandDir, TEXT("del"), 0, szTmpFile);
    ATLASSERT(nRet);

    if(!nRet) return false;

    BOOL bRes = ::MoveFileEx(deskbandPath, szTmpFile, MOVEFILE_REPLACE_EXISTING);
    ATLASSERT(bRes);

    if(!bRes) return false;

    // If user is an admin, then the file gets deleted on the next reboot;
    // for standard users the MOVEFILE_DELAY_UNTIL_REBOOT flag is unavailable,
    // so we delete with DeleteFile, which is likely to fail because
    // the deskband DLL will be in use by Explorer.exe. However, the file
    // will be deleted when last handle to it is closed.
    if(::IsUserAnAdmin())
    {
        bRes = ::MoveFileEx(szTmpFile, NULL, MOVEFILE_DELAY_UNTIL_REBOOT);
    }
    else
    {
        ::DeleteFile(szTmpFile);
        bRes = TRUE;
    }
    ATLASSERT(bRes);

    return (bRes != FALSE);
}

bool CVDSetup::RegisterDeskband(const CPath& deskbandPath, bool bRegister) const
{
    bool bRes = false;
    if(::IsVistaOrHigher())
    {
        bRes = (RegisterDeskbandSecure(deskbandPath, bRegister) == ERROR_SUCCESS);
    }
    else    // XP
    {
        bRes = SUCCEEDED(RegisterDeskbandXP(deskbandPath, bRegister));
    }

    return bRes;
}

struct LibraryLoader
{
    LibraryLoader(LPCTSTR pszPath) : hModule(::LoadLibrary(pszPath)) {}
    ~LibraryLoader() { if(hModule) ::FreeLibrary(hModule); }

    operator HMODULE() const { return hModule; }

    HMODULE hModule;
};

HRESULT CVDSetup::RegisterDeskbandXP(const CPath& deskbandPath, bool bRegister) const
{
    LibraryLoader deskbandLib(deskbandPath);

    HRESULT hr = E_FAIL;
    if(deskbandLib)
    {
        typedef HRESULT (STDAPICALLTYPE *REGISTERSERVERPROC)(void);

        const LPCSTR pszProcName = (bRegister ?
            "DllRegisterServer" : "DllUnregisterServer");

        REGISTERSERVERPROC DllRegisterProc =
            reinterpret_cast<REGISTERSERVERPROC>(
                ::GetProcAddress(deskbandLib, pszProcName));

        if(DllRegisterProc)
        {
            hr = DllRegisterProc();
            ATLASSERT(SUCCEEDED(hr));

            return hr;
        }
    }

    hr = HRESULT_FROM_WIN32(::GetLastError());
    ATLASSERT(FALSE);

    return hr;
}

DWORD CVDSetup::RegisterDeskbandSecure(CPath deskbandPath, bool bRegister) const
{
    deskbandPath.QuoteSpaces();
    const CString& parameters = (bRegister ? TEXT("") : TEXT("/u ")) + deskbandPath;

    const DWORD dwExitCode = ::RunAsAdmin(m_hWnd, TEXT("REGSVR32.EXE"), parameters);
    ATLASSERT(dwExitCode == ERROR_SUCCESS);

    return dwExitCode;
}

bool CVDSetup::ShowDeskband() const
{
    CComPtr<ITrayDeskBand> spTrayDeskBand;
    HRESULT hr = spTrayDeskBand.CoCreateInstance(CLSID_TrayDeskBand);

    if(SUCCEEDED(hr))   // Vista and higher
    {
        hr = spTrayDeskBand->DeskBandRegistrationChanged();
        ATLASSERT(SUCCEEDED(hr));

        if(SUCCEEDED(hr))
        {
            hr = spTrayDeskBand->IsDeskBandShown(CLSID_VolumeDeskBand);
            ATLASSERT(SUCCEEDED(hr));

            if(SUCCEEDED(hr) && hr == S_FALSE)
                hr = spTrayDeskBand->ShowDeskBand(CLSID_VolumeDeskBand);
        }
    }
    else    // WinXP workaround
    {
        const CString& sAtom = ::StringFromGUID2(CLSID_VolumeDeskBand);

        if(!::GlobalFindAtom(sAtom))
            ::GlobalAddAtom(sAtom);

        // Beware! SHLoadInProc is not implemented under Vista and higher.
        hr = ::SHLoadInProc(CLSID_VolumeDeskBand);
        ATLASSERT(SUCCEEDED(hr));
    }

    return SUCCEEDED(hr);
}

bool CVDSetup::HideDeskband() const
{
    CComPtr<ITrayDeskBand> spTrayDeskBand;
    HRESULT hr = spTrayDeskBand.CoCreateInstance(CLSID_TrayDeskBand);

    if(SUCCEEDED(hr))   // Vista and higher
    {
        hr = spTrayDeskBand->IsDeskBandShown(CLSID_VolumeDeskBand);

        if(hr == S_OK)
            hr = spTrayDeskBand->HideDeskBand(CLSID_VolumeDeskBand);
    }
    else    // WinXP
    {
        CComPtr<IBandSite> spBandSite;
        hr = spBandSite.CoCreateInstance(CLSID_TrayBandSiteService);

        if(SUCCEEDED(hr))
        {
            DWORD dwBandID = 0;
            const UINT nBandCount = spBandSite->EnumBands((UINT)-1, &dwBandID);

            for(UINT i = 0; i < nBandCount; ++i)
            {
                spBandSite->EnumBands(i, &dwBandID);

                CComPtr<IPersist> spPersist;
                hr = spBandSite->GetBandObject(dwBandID, IID_IPersist, (void**)&spPersist);
                if(SUCCEEDED(hr))
                {
                    CLSID clsid = CLSID_NULL;
                    hr = spPersist->GetClassID(&clsid);

                    if(SUCCEEDED(hr) && ::IsEqualCLSID(clsid, CLSID_VolumeDeskBand))
                    {
                        hr = spBandSite->RemoveBand(dwBandID);
                        break;
                    }
                }
            }
        }
    }

    return SUCCEEDED(hr);
}

